@@ -130,7 +130,7 @@ static PyObject *py_creds_get_ntlm_response(PyObject *self, PyObject *args, PyOb
 	ret = Py_BuildValue("{sis" PYARG_BYTES_LEN "s" PYARG_BYTES_LEN
 			            "s" PYARG_BYTES_LEN "s" PYARG_BYTES_LEN "}",
 			    "flags", flags,
-			    "lm_reponse",
+			    "lm_response",
 			    (const char *)lm_response.data, lm_response.length,
 			    "nt_response",
 			    (const char *)nt_response.data, nt_response.length,
@@ -171,16 +171,18 @@ static PyObject *py_creds_get_password(PyObject *self, PyObject *unused)
 
 static PyObject *py_creds_set_password(PyObject *self, PyObject *args)
 {
-	char *newval;
+	const char *newval = NULL;
 	enum credentials_obtained obt = CRED_SPECIFIED;
 	int _obt = obt;
-
-	if (!PyArg_ParseTuple(args, "s|i", &newval, &_obt)) {
+	PyObject *result = NULL;
+	if (!PyArg_ParseTuple(args, PYARG_STR_UNI"|i", "utf8", &newval, &_obt)) {
 		return NULL;
 	}
 	obt = _obt;
 
-	return PyBool_FromLong(cli_credentials_set_password(PyCredentials_AsCliCredentials(self), newval, obt));
+	result = PyBool_FromLong(cli_credentials_set_password(PyCredentials_AsCliCredentials(self), newval, obt));
+	PyMem_Free(discard_const_p(void*, newval));
+	return result;
 }
 
 static PyObject *py_creds_set_utf16_password(PyObject *self, PyObject *args)
@@ -659,7 +661,7 @@ static PyObject *py_creds_new_client_authenticator(PyObject *self,
 	netlogon_creds_client_authenticator(
 		nc,
 		&auth);
-	ret = Py_BuildValue("{ss#si}",
+	ret = Py_BuildValue("{s"PYARG_BYTES_LEN"si}",
 			    "credential",
 			    (const char *) &auth.cred, sizeof(auth.cred),
 			    "timestamp", auth.timestamp);
@@ -680,6 +682,16 @@ static PyObject *py_creds_set_secure_channel_type(PyObject *self, PyObject *args
 	Py_RETURN_NONE;
 }
 
+static PyObject *py_creds_get_secure_channel_type(PyObject *self, PyObject *args)
+{
+	enum netr_SchannelType channel_type = SEC_CHAN_NULL;
+
+	channel_type = cli_credentials_get_secure_channel_type(
+		PyCredentials_AsCliCredentials(self));
+
+	return PyInt_FromLong(channel_type);
+}
+
 static PyObject *py_creds_encrypt_netr_crypt_password(PyObject *self,
 						      PyObject *args)
 {
@@ -815,6 +827,8 @@ static PyMethodDef py_creds_methods[] = {
 		"Get a new client NETLOGON_AUTHENTICATOR"},
 	{ "set_secure_channel_type", py_creds_set_secure_channel_type,
 	  METH_VARARGS, NULL },
+	{ "get_secure_channel_type", py_creds_get_secure_channel_type,
+	  METH_VARARGS },
 	{ "encrypt_netr_crypt_password",
 		py_creds_encrypt_netr_crypt_password,
 		METH_VARARGS,
