@@ -78,11 +78,7 @@ static void fault_report(int sig)
 
 	DEBUGSEP(0);
 	DEBUG(0,("INTERNAL ERROR: Signal %d in pid %d (%s)",sig,(int)getpid(),SAMBA_VERSION_STRING));
-	DEBUG(0,("\nIf you are running a recent Samba version, and "
-		 "if you think this problem is not yet fixed in the "
-		 "latest versions, please consider reporting this "
-		 "bug, see "
-		 "https://wiki.samba.org/index.php/Bug_Reporting\n"));
+	DEBUG(0,("\nPlease read the Trouble-Shooting section of the Samba HOWTO\n"));
 	DEBUGSEP(0);
 
 	smb_panic("internal error");
@@ -252,7 +248,7 @@ void log_stack_trace(void)
 libunwind_failed:
 	DEBUG(0, ("unable to produce a stack trace with libunwind\n"));
 
-#elif HAVE_BACKTRACE_SYMBOLS
+#elif defined(HAVE_BACKTRACE_SYMBOLS)
 	void *backtrace_stack[BACKTRACE_STACK_SIZE];
 	size_t backtrace_size;
 	char **backtrace_strings;
