@@ -159,29 +159,34 @@ static struct dptr_struct *dptr_get(struct smbd_server_connection *sconn,
 				    int key, bool forclose)
 {
 	struct dptr_struct *dptr;
+	const int dirhandles_open = sconn->searches.dirhandles_open;
 
-	for(dptr = sconn->searches.dirptrs; dptr; dptr = dptr->next) {
-		if(dptr->dnum == key) {
-			if (!forclose && !dptr->dir_hnd) {
-				if (sconn->searches.dirhandles_open >= MAX_OPEN_DIRECTORIES)
-					dptr_idleoldest(sconn);
-				DEBUG(4,("dptr_get: Reopening dptr key %d\n",key));
-
-				if (!(dptr->dir_hnd = OpenDir(NULL,
-							dptr->conn,
-							dptr->smb_dname,
-							dptr->wcard,
-							dptr->attr))) {
-					DEBUG(4,("dptr_get: Failed to "
-						"open %s (%s)\n",
-						dptr->smb_dname->base_name,
-						strerror(errno)));
-					return NULL;
-				}
-			}
-			DLIST_PROMOTE(sconn->searches.dirptrs,dptr);
-			return dptr;
+	for (dptr = sconn->searches.dirptrs; dptr != NULL; dptr = dptr->next) {
+		if(dptr->dnum != key) {
+			continue;
 		}
+
+		if (!forclose && (dptr->dir_hnd == NULL)) {
+			if (dirhandles_open >= MAX_OPEN_DIRECTORIES) {
+				dptr_idleoldest(sconn);
+			}
+			DBG_INFO("Reopening dptr key %d\n",key);
+
+			dptr->dir_hnd = OpenDir(NULL,
+						dptr->conn,
+						dptr->smb_dname,
+						dptr->wcard,
+						dptr->attr);
+
+			if (dptr->dir_hnd == NULL) {
+				DBG_INFO("Failed to open %s (%s)\n",
+				      dptr->smb_dname->base_name,
+				      strerror(errno));
+				return NULL;
+			}
+		}
+		DLIST_PROMOTE(sconn->searches.dirptrs, dptr);
+		return dptr;
 	}
 	return(NULL);
 }
@@ -1082,6 +1087,7 @@ bool smbd_dirptr_get_entry(TALLOC_CTX *ctx,
 			   uint32_t dirtype,
 			   bool dont_descend,
 			   bool ask_sharemode,
+			   bool get_dosmode,
 			   bool (*match_fn)(TALLOC_CTX *ctx,
 					    void *private_data,
 					    const char *dname,
@@ -1090,6 +1096,7 @@ bool smbd_dirptr_get_entry(TALLOC_CTX *ctx,
 			   bool (*mode_fn)(TALLOC_CTX *ctx,
 					   void *private_data,
 					   struct smb_filename *smb_fname,
+					   bool get_dosmode,
 					   uint32_t *_mode),
 			   void *private_data,
 			   char **_fname,
@@ -1184,7 +1191,7 @@ bool smbd_dirptr_get_entry(TALLOC_CTX *ctx,
 			.base_name = pathreal, .st = sbuf
 		};
 
-		ok = mode_fn(ctx, private_data, &smb_fname, &mode);
+		ok = mode_fn(ctx, private_data, &smb_fname, get_dosmode, &mode);
 		if (!ok) {
 			TALLOC_FREE(dname);
 			TALLOC_FREE(fname);
@@ -1313,6 +1320,7 @@ static bool smbd_dirptr_8_3_match_fn(TALLOC_CTX *ctx,
 static bool smbd_dirptr_8_3_mode_fn(TALLOC_CTX *ctx,
 				    void *private_data,
 				    struct smb_filename *smb_fname,
+				    bool get_dosmode,
 				    uint32_t *_mode)
 {
 	connection_struct *conn = (connection_struct *)private_data;
@@ -1356,6 +1364,7 @@ bool get_dir_entry(TALLOC_CTX *ctx,
 				   dirtype,
 				   check_descend,
 				   ask_sharemode,
+				   true,
 				   smbd_dirptr_8_3_match_fn,
 				   smbd_dirptr_8_3_mode_fn,
 				   conn,
@@ -1527,6 +1536,7 @@ bool is_visible_file(connection_struct *conn, const char *dir_path,
 	bool hide_unreadable = lp_hide_unreadable(SNUM(conn));
 	bool hide_unwriteable = lp_hide_unwriteable_files(SNUM(conn));
 	bool hide_special = lp_hide_special_files(SNUM(conn));
+	int hide_new_files_timeout = lp_hide_new_files_timeout(SNUM(conn));
 	char *entry = NULL;
 	struct smb_filename *smb_fname_base = NULL;
 	bool ret = false;
@@ -1541,7 +1551,11 @@ bool is_visible_file(connection_struct *conn, const char *dir_path,
 		return False;
 	}
 
-	if (hide_unreadable || hide_unwriteable || hide_special) {
+	if (hide_unreadable ||
+	    hide_unwriteable ||
+	    hide_special ||
+	    (hide_new_files_timeout != 0))
+	{
 		entry = talloc_asprintf(talloc_tos(), "%s/%s", dir_path, name);
 		if (!entry) {
 			ret = false;
@@ -1594,6 +1608,17 @@ bool is_visible_file(connection_struct *conn, const char *dir_path,
 			ret = false;
 			goto out;
 		}
+
+		if (hide_new_files_timeout != 0) {
+
+			double age = timespec_elapsed(
+				&smb_fname_base->st.st_ex_mtime);
+
+			if (age < (double)hide_new_files_timeout) {
+				ret = false;
+				goto out;
+			}
+		}
 	}
 
 	ret = true;
@@ -2049,24 +2074,29 @@ static int files_below_forall_fn(struct file_id fid,
 		/*
 		 * Filter files above dirpath
 		 */
-		return 0;
+		goto out;
 	}
 	if (fullpath[state->dirpath_len] != '/') {
 		/*
 		 * Filter file that don't have a path separator at the end of
 		 * dirpath's length
 		 */
-		return 0;
+		goto out;
 	}
 
 	if (memcmp(state->dirpath, fullpath, state->dirpath_len) != 0) {
 		/*
 		 * Not a parent
 		 */
-		return 0;
+		goto out;
 	}
 
+	TALLOC_FREE(to_free);
 	return state->fn(fid, data, state->private_data);
+
+out:
+	TALLOC_FREE(to_free);
+	return 0;
 }
 
 static int files_below_forall(connection_struct *conn,
