@@ -47,8 +47,9 @@
 #include "nsswitch/libwbclient/wbclient.h"
 #include "lib/param/loadparm.h"
 #include "lib/util/base64.h"
+#include "cmdline_contexts.h"
 
-#if HAVE_KRB5
+#ifdef HAVE_KRB5
 #include "auth/kerberos/pac_utils.h"
 #endif
 
@@ -2380,6 +2381,8 @@ enum {
 
 	poptFreeContext(pc);
 
+	cmdline_messaging_context(get_dyn_CONFIGFILE());
+
 	if (!lp_load_global(get_dyn_CONFIGFILE())) {
 		d_fprintf(stderr, "ntlm_auth: error opening config file %s. Error was %s\n",
 			get_dyn_CONFIGFILE(), strerror(errno));
