@@ -601,7 +601,7 @@ static PyObject *py_smb_new(PyTypeObject *type, PyObject *args, PyObject *kwargs
 	PyObject *py_creds = Py_None;
 	PyObject *py_lp = Py_None;
 	const char *kwnames[] = { "hostname", "service", "creds", "lp",
-				  "ntlmv2_auth", "use_spnego", NULL };
+				  "ntlmv2_auth", "use_spnego", "sign", NULL };
 	const char *hostname = NULL;
 	const char *service = NULL;
 	PyObject *smb;
@@ -612,11 +612,24 @@ static PyObject *py_smb_new(PyTypeObject *type, PyObject *args, PyObject *kwargs
 	struct smbcli_session_options session_options;
 	uint8_t ntlmv2_auth = 0xFF;
 	uint8_t use_spnego = 0xFF;
+	PyObject *sign = Py_False;
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "zz|OObb",
+	/*
+	 * These Python bindings are now deprecated because the s4 SMB client
+	 * code doesn't support SMBv2 (and is unlikely to ever support it).
+	 * The s3 libsmb_samba_internal bindings are a better choice for use
+	 * within the Samba codebase, and support much the same API.
+	 * This warning is mostly for external consumers that might be using
+	 * these Python bindings (in which case, note libsmb_samba_internal
+	 * is not a stable API and may change in future).
+	 */
+	DBG_ERR("The smb.SMB() Python bindings are now deprecated "
+		"and will be removed in the next samba release\n");
+
+	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "zz|OObbO",
 					 discard_const_p(char *, kwnames),
 					 &hostname, &service, &py_creds, &py_lp,
-					 &ntlmv2_auth, &use_spnego)) {
+					 &ntlmv2_auth, &use_spnego, &sign)) {
 		return NULL;
 	}
 
@@ -658,6 +671,9 @@ static PyObject *py_smb_new(PyTypeObject *type, PyObject *args, PyObject *kwargs
 	if (use_spnego != 0xFF) {
 		options.use_spnego = use_spnego;
 	}
+	if (PyObject_IsTrue(sign)) {
+		options.signing = SMB_SIGNING_REQUIRED;
+	}
 
 	status = do_smb_connect(spdata, spdata, hostname, service,
 				&options,
