@@ -51,7 +51,7 @@ static bool test_eas(struct smbcli_state *cli, struct torture_context *tctx)
 	union smb_open io;
 	const char *fname = BASEDIR "\\ea.txt";
 	bool ret = true;
-	char bad_ea_name[6];
+	char bad_ea_name[7];
 	int i;
 	int fnum = -1;
 
@@ -304,7 +304,7 @@ static bool test_max_eas(struct smbcli_state *cli, struct torture_context *tctx)
 	maxeasize  = torture_setting_int(tctx, "maxeasize", 65536);
 	maxeanames = torture_setting_int(tctx, "maxeanames", 101);
 	maxeastart = torture_setting_int(tctx, "maxeastart", 1);
-	maxeadebug = torture_setting_int(tctx, "maxeadebug", 0);
+	maxeadebug = torture_setting_bool(tctx, "maxeadebug", false);
 
 	/* Do some sanity check on possibly passed parms */
 	if (maxeasize <= 0) {
@@ -319,10 +319,6 @@ static bool test_max_eas(struct smbcli_state *cli, struct torture_context *tctx)
 		torture_comment(tctx, "Invalid parameter 'maxeastart=%d'",maxeastart);
 		err = true;
 	}
-	if (maxeadebug < 0) {
-		torture_comment(tctx, "Invalid parameter 'maxeadebug=%d'",maxeadebug);
-		err = true;
-	}
 	if (err) {
 	  torture_comment(tctx, "\n\n");
 	  goto done;
